'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard   www.joseng.de                                       |
'| Datei: DMX-Relaisplatine.bas                                                |
'| Beschreibung: DMX Relaisplatine 2fach ohne Extras                           |
'|                                                                             |
'| Prozessor: ATtiny2313                                                       |
'| Fuse Bits:  LowByte: 0xFF    HighByte: 0xDF    EByte: 0xFF                  |
'|                                                                             |
'+-----------------------------------------------------------------------------+

$regfile = "attiny2313.dat"                                 'ATtiny2313
$crystal = 16000000                                         '16MHz Quarz
$baud = 250000                                              'Baudrate 250Kbaud

$hwstack = 40                                               'Jeder Gosub bentigt 2 Byte, bei verschachtellungen also 2+2...
                                                             'Ein Interrupt bentigt hier alleine 32 Byte
$swstack = 10                                               'Jeder bergebene Parameter oder jede lokale Variable braucht 2 Byte hier
$framesize = 24                                             'Byval bergebene Werte werden hier rein gespeichert


Config Portb = Input                                        'Dip Switch fr Adresse
Portb = 255                                                 'PullUps einschalten
Config Portd.1 = Input                                      'Adressbit 8
Set Portd.1                                                 'Pullup ein

Config Portd.2 = Output                                     'Relais 1
Config Portd.3 = Output                                     'Relais 2

Config Portd.6 = Output                                     'StatusLED

Config Portd.4 = Input                                      'Jumper 1
Config Portd.5 = Input                                      'Jumper 2
Set Portd.4                                                 'PullUp einschalten
Set Portd.5                                                 'PullUp einschalten

'Funktionsvariablen
Dim Adresse As Word

'DMX Variablen
Dim Dmx_count As Word                                       'Pakete die auf Ausgnge zielen
Dim Dmx_count_temp As Byte                                  'Gesendete Pakete
Dim Dmx_byte As Byte                                        'DMX Wert
Dim Dmx_clock As Byte                                       'DMX-Led
Dim Dmx_adr As Word                                         'DMX-Adresse
Dim Dmx_adr_temp As Byte                                    'DMX-Adresse (Temp)
Dim Dmx_adr_c As Word                                       'DMX-Adresse (Counter)
Dim Dmx_off As Byte                                         'DMX-Signal noch da?

'Statusled setzen
Set Portd.6

'Relais ausschalten
Reset Portd.2
Reset Portd.3


' Uart initialiesieren
On Urxc Rec_isr
Enable Urxc

'UART TX Pin ausschalten
Reset Ucsrb.txen

' Anfangsvariablen Festlegen
Dmx_count = 0                                               'kein Paket empfangen
Dmx_clock = 16                                              '

'Interrupts freischalten
Enable Interrupts

'Hauptprogramm
Do
  Waitms 10
  Decr Dmx_off
  If Dmx_off = 0 Then
    Reset Portd.6
  End If

  Adresse = 255 - Pinb                                      'Adressbit 0-7 einlesen
  If Pind.1 = 0 Then Adresse = Adresse + 256                'Adressbit 8 einlesen
  Dmx_adr = Adresse
Loop

' ******************************************************************************
' Funktion: DMX-Signal emfangen und auswerten
'           Dieser Interupt wird beim eintreffen von seriellen Daten ausgefhrt
Rec_isr:
   $asm                                                     'Programmteil fr das Auswerten der
      .def Uart_status = R16                                'emfangenen Daten
      .def Uart_byte = R17
      .def Uart_temp1 = R18

            IN Uart_status,USR
            IN Uart_byte,UDR

            SBRC Uart_status,FE                             'Auswertung ob Datenbyte OK
            RJMP  Asm_reset                                 'bei berlnge des Datenbyte = DMX Reset
            RJMP  Asm_Loop1                                 'Daten OK--->

      Asm_reset:
            LDI Uart_temp1,$00                              'Adesszhler zurcksetzen
            STS {Dmx_count_temp} , Uart_temp1               'Daten dem Basicvariablen bergeben
            STS {Dmx_byte} , Uart_byte
            RJMP Asm_exit
      Asm_loop1:
            STS {Dmx_byte} , Uart_byte
      Asm_exit:
   $end Asm

   If Dmx_count_temp <> 0 Then                              'emfangene Daten im "Dmx_buf" ablegen
      Incr Dmx_count                                        'und den jeweiligen Kanal zuordnen
      If Dmx_adr_c = Dmx_count Then
         If Dmx_adr_temp < 3 Then
            Select Case Dmx_adr_temp
               Case 1:
                  If 127 <= Dmx_byte Then : Set Portd.2 : Else : Reset Portd.2 : End If
               Case 2:
                  If 127 <= Dmx_byte Then : Set Portd.3 : Else : Reset Portd.3 : End If
            End Select
            Incr Dmx_adr_temp
            Incr Dmx_adr_c
         End If
      End If
   Else                                                     'bei DMX Reset Variablen setzen
      Dmx_count = 0 : Dmx_count_temp = 1
      Dmx_adr_c = Dmx_adr + 1 : Dmx_adr_temp = 1
      Decr Dmx_clock
      'nur eine Spielerei Toggelt eine LED am PortD.6 beim eintreffen von korrekten DMX Daten
      If Dmx_clock = 0 Then : Toggle Portd.6 : Dmx_clock = 16 : End If
   End If
   Dmx_off = 64
Return